@@ -1,6 +1,11 @@
 # -*- coding: utf-8 -*-
+"""
+Wrappers around API results to provide more convenient attribute access and IIIF helpers.
+"""
+
 import tempfile
 from contextlib import contextmanager
+from typing import Optional
 
 from requests import HTTPError
 
@@ -36,6 +41,9 @@ class MagicDict(dict):
                 "{} object has no attribute '{}'".format(self.__class__.__name__, name)
             )
 
+    def __setattr__(self, name, value):
+        return super().__setitem__(name, value)
+
     def __delattr__(self, name):
         try:
             return super().__delattr__(name)
@@ -52,13 +60,24 @@ class MagicDict(dict):
 
 class Element(MagicDict):
     """
-    Describes any kind of element.
+    Describes an Arkindex element.
     """
 
-    def image_url(self, size="full"):
+    def resize_zone_url(self, size="full"):
+        if size == "full":
+            return self.zone.url
+        else:
+            parts = self.zone.url.split("/")
+            parts[-3] = size
+            return "/".join(parts)
+
+    def image_url(self, size="full") -> Optional[str]:
         """
-        When possible, will return the S3 URL for images, so an ML worker can bypass IIIF servers
-        :param size: Subresolution of the image.
+        Build an URL to access the image.
+        When possible, will return the S3 URL for images, so an ML worker can bypass IIIF servers.
+        :param str size: Subresolution of the image, following the syntax of the IIIF resize parameter.
+        :returns: An URL to the image, or None if the element does not have an image.
+        :rtype: str or None
         """
         if not self.get("zone"):
             return
@@ -71,7 +90,11 @@ class Element(MagicDict):
         return "{}full/{}/0/default.jpg".format(url, size)
 
     @property
-    def requires_tiles(self):
+    def requires_tiles(self) -> bool:
+        """
+        :return bool: Whether or not downloading and combining IIIF tiles will be necessary
+           to retrieve this element's image. Will be False if the element has no image.
+        """
         if not self.get("zone") or self.zone.image.get("s3_url"):
             return False
         max_width = self.zone.image.server.max_width or float("inf")
@@ -79,10 +102,41 @@ class Element(MagicDict):
         bounding_box = polygon_bounding_box(self.zone.polygon)
         return bounding_box.width > max_width or bounding_box.height > max_height
 
-    def open_image(self, *args, max_size=None, **kwargs):
+    def open_image(self, *args, max_size=None, use_full_image=False, **kwargs):
         """
-        Open this element's image as a Pillow image.
-        :param max_size: Subresolution of the image.
+        Open this element's image using Pillow, rotating and mirroring it according
+        to the ``rotation_angle`` and ``mirrored`` attributes.
+
+        When tiling is not required to download the image, and no S3 URL is available
+        to bypass IIIF servers, the image will be cropped to the rectangle bounding box
+        of the ``zone.polygon`` attribute.
+
+        .. warning::
+
+           This method implicitly applies the element's orientation to the image.
+
+           If your process uses the returned image to find more polygons and send them
+           back to Arkindex, use the :meth:`arkindex_worker.image.revert_orientation`
+           helper to undo the orientation on all polygons before sending them, as the
+           Arkindex API expects unoriented polygons.
+
+           Although not recommended, you can bypass this behavior by passing
+           ``rotation_angle=0, mirrored=False`` as keyword arguments.
+
+
+        :param max_size: The maximum size of the requested image.
+        :type max_size: int or None
+        :param bool use_full_image: Ignore the ``zone.polygon`` and always
+           retrieve the image without cropping.
+        :param \\*args: Positional arguments passed to :meth:`arkindex_worker.image.open_image`.
+        :param \\**kwargs: Keyword arguments passed to :meth:`arkindex_worker.image.open_image`.
+        :raises ValueError: When the element does not have an image.
+        :raises NotImplementedError: When the ``max_size`` parameter is set,
+           but the IIIF server's configuration requires downloading and combining tiles
+           to retrieve the image.
+        :raises NotImplementedError: When an S3 URL has been used to download the image,
+           but the URL has expired. Re-fetching the URL automatically is not supported.
+        :return PIL.Image: A Pillow image.
         """
         if not self.get("zone"):
             raise ValueError("Element {} has no zone".format(self.id))
@@ -117,8 +171,19 @@ class Element(MagicDict):
         else:
             resize = "full"
 
+        if use_full_image:
+            url = self.image_url(resize)
+        else:
+            url = self.resize_zone_url(resize)
+
         try:
-            return open_image(self.image_url(resize), *args, **kwargs)
+            return open_image(
+                url,
+                *args,
+                rotation_angle=self.rotation_angle,
+                mirrored=self.mirrored,
+                **kwargs
+            )
         except HTTPError as e:
             if (
                 self.zone.image.get("s3_url") is not None
@@ -135,7 +200,15 @@ class Element(MagicDict):
     def open_image_tempfile(self, format="jpeg", *args, **kwargs):
         """
         Get the element's image as a temporary file stored on the disk.
-        To be used as a context manager: with element.open_image_tempfile() as f: ...
+        To be used as a context manager::
+
+           with element.open_image_tempfile() as f:
+               ...
+
+        :param format str: File format to use the store the image on the disk.
+           Must be a format supported by Pillow.
+        :param \\*args: Positional arguments passed to :meth:`arkindex_worker.image.open_image`.
+        :param \\**kwargs: Keyword arguments passed to :meth:`arkindex_worker.image.open_image`.
         """
         with tempfile.NamedTemporaryFile() as f:
             self.open_image(*args, **kwargs).save(f, format=format)
