@@ -12,7 +12,7 @@ from arkindex_worker import logger
 from arkindex_worker.worker import BaseWorker
 
 
-def test_init_default_local_share():
+def test_init_default_local_share(monkeypatch):
     worker = BaseWorker()
 
     assert worker.work_dir == os.path.expanduser("~/.local/share/arkindex")
@@ -28,6 +28,14 @@ def test_init_default_xdg_data_home(monkeypatch):
     assert worker.worker_version_id == "12341234-1234-1234-1234-123412341234"
 
 
+def test_init_with_local_cache(monkeypatch):
+    worker = BaseWorker(support_cache=True)
+
+    assert worker.work_dir == os.path.expanduser("~/.local/share/arkindex")
+    assert worker.worker_version_id == "12341234-1234-1234-1234-123412341234"
+    assert worker.support_cache is True
+
+
 def test_init_var_ponos_data_given(monkeypatch):
     path = str(Path(__file__).absolute().parent)
     monkeypatch.setenv("PONOS_DATA", path)
@@ -37,7 +45,9 @@ def test_init_var_ponos_data_given(monkeypatch):
     assert worker.worker_version_id == "12341234-1234-1234-1234-123412341234"
 
 
-def test_init_var_worker_version_id_missing(monkeypatch, mock_user_api):
+def test_init_var_worker_version_id_missing(
+    monkeypatch, mock_user_api, mock_process_api
+):
     monkeypatch.setattr(sys, "argv", ["worker"])
     monkeypatch.delenv("WORKER_VERSION_ID")
     worker = BaseWorker()
@@ -47,7 +57,9 @@ def test_init_var_worker_version_id_missing(monkeypatch, mock_user_api):
     assert worker.config == {}  # default empty case
 
 
-def test_init_var_worker_local_file(monkeypatch, tmp_path, mock_user_api):
+def test_init_var_worker_local_file(
+    monkeypatch, tmp_path, mock_user_api, mock_process_api
+):
     # Build a dummy yaml config file
     config = tmp_path / "config.yml"
     config.write_text("---\nlocalKey: abcdef123")
@@ -63,7 +75,7 @@ def test_init_var_worker_local_file(monkeypatch, tmp_path, mock_user_api):
     config.unlink()
 
 
-def test_cli_default(mocker, mock_worker_version_api, mock_user_api):
+def test_cli_default(mocker, mock_config_api):
     worker = BaseWorker()
     spy = mocker.spy(worker, "add_arguments")
     assert not spy.called
@@ -85,7 +97,7 @@ def test_cli_default(mocker, mock_worker_version_api, mock_user_api):
     logger.setLevel(logging.NOTSET)
 
 
-def test_cli_arg_verbose_given(mocker, mock_worker_version_api, mock_user_api):
+def test_cli_arg_verbose_given(mocker, mock_config_api):
     worker = BaseWorker()
     spy = mocker.spy(worker, "add_arguments")
     assert not spy.called
@@ -107,6 +119,96 @@ def test_cli_arg_verbose_given(mocker, mock_worker_version_api, mock_user_api):
     logger.setLevel(logging.NOTSET)
 
 
+def test_configure_dev_mode(
+    mocker, monkeypatch, mock_user_api, mock_worker_version_api
+):
+    """
+    Configuring a worker in developer mode avoid retrieving process information
+    """
+    worker = BaseWorker()
+    mocker.patch.object(sys, "argv", ["worker", "--dev"])
+    monkeypatch.setenv(
+        "ARKINDEX_WORKER_RUN_ID", "aaaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"
+    )
+    worker.configure()
+
+    assert worker.args.dev is True
+    assert worker.process_information is None
+    assert worker.worker_version_id == "12341234-1234-1234-1234-123412341234"
+    assert worker.is_read_only is True
+    assert worker.user_configuration == {}
+
+
+def test_configure_worker_run(mocker, monkeypatch, responses, mock_config_api):
+    worker = BaseWorker()
+    mocker.patch.object(sys, "argv", ["worker"])
+    run_id = "aaaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"
+    configuration_id = "bbbbbbbbb-bbbb-bbbb-bbbb-bbbbbbbbbbbb"
+    monkeypatch.setenv("ARKINDEX_WORKER_RUN_ID", run_id)
+    responses.add(
+        responses.GET,
+        f"http://testserver/api/v1/imports/workers/{run_id}/",
+        json={"id": run_id, "configuration_id": configuration_id},
+    )
+    responses.add(
+        responses.GET,
+        f"http://testserver/api/v1/workers/configurations/{configuration_id}/",
+        json={"id": configuration_id, "name": "BBB", "configuration": {"a": "b"}},
+    )
+    worker.configure()
+
+    assert worker.user_configuration == {"a": "b"}
+
+
+def test_configure_user_configuration_defaults(
+    mocker,
+    monkeypatch,
+    mock_worker_version_user_configuration_api,
+    mock_user_api,
+    mock_process_api,
+    responses,
+):
+    worker = BaseWorker()
+    mocker.patch.object(sys, "argv")
+    run_id = "aaaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"
+    monkeypatch.setenv("ARKINDEX_WORKER_RUN_ID", run_id)
+    responses.add(
+        responses.GET,
+        f"http://testserver/api/v1/imports/workers/{run_id}/",
+        json={"id": run_id},
+    )
+    worker.configure()
+
+    assert worker.config == {"param_1": "/some/path/file.pth", "param_2": 12}
+    assert worker.user_configuration == {
+        "param_3": "Animula vagula blandula",
+        "param_5": True,
+    }
+
+
+def test_configure_worker_run_missing_conf(
+    mocker, monkeypatch, responses, mock_config_api
+):
+    worker = BaseWorker()
+    mocker.patch.object(sys, "argv", ["worker"])
+    run_id = "aaaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa"
+    configuration_id = "bbbbbbbbb-bbbb-bbbb-bbbb-bbbbbbbbbbbb"
+    monkeypatch.setenv("ARKINDEX_WORKER_RUN_ID", run_id)
+    responses.add(
+        responses.GET,
+        f"http://testserver/api/v1/imports/workers/{run_id}/",
+        json={"id": run_id, "configuration_id": configuration_id},
+    )
+    responses.add(
+        responses.GET,
+        f"http://testserver/api/v1/workers/configurations/{configuration_id}/",
+        json={"id": configuration_id, "name": "BBB"},
+    )
+    worker.configure()
+
+    assert worker.user_configuration is None
+
+
 def test_load_missing_secret():
     worker = BaseWorker()
     worker.api_client = MockApiClient()
